@@ -22,6 +22,7 @@ void bind_peer_plugin();
 void bind_torrent();
 void bind_peer_info();
 void bind_ip_filter();
+void bind_magnet_uri();
 
 BOOST_PYTHON_MODULE(libtorrent)
 {
@@ -46,5 +47,6 @@ BOOST_PYTHON_MODULE(libtorrent)
     bind_torrent();
     bind_peer_info();
     bind_ip_filter();
+    bind_magnet_uri();
 }
 
