@@ -40,6 +40,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket.hpp"
 #include "libtorrent/time.hpp"
 
+#ifdef TORRENT_CONNECTION_LOGGING
+#include <fstream>
+#endif
+
 namespace libtorrent
 {
 
@@ -48,30 +52,31 @@ class connection_queue : public boost::noncopyable
 public:
 	connection_queue(io_service& ios);
 
-	bool free_slots() const;
+	// if there are no free slots, returns the negative
+	// number of queued up connections
+	int free_slots() const;
 
 	void enqueue(boost::function<void(int)> const& on_connect
 		, boost::function<void()> const& on_timeout
-		, time_duration timeout);
+		, time_duration timeout, int priority = 0);
 	void done(int ticket);
 	void limit(int limit);
 	int limit() const;
 	void close();
+	int size() const { return m_queue.size(); }
 
 #ifndef NDEBUG
-
 	void check_invariant() const;
-
 #endif
 
 private:
 
 	void try_connect();
-	void on_timeout(asio::error_code const& e);
+	void on_timeout(error_code const& e);
 
 	struct entry
 	{
-		entry(): connecting(false), ticket(0), expires(max_time()) {}
+		entry(): connecting(false), ticket(0), expires(max_time()), priority(0) {}
 		// called when the connection is initiated
 		boost::function<void(int)> on_connect;
 		// called if done hasn't been called within the timeout
@@ -80,6 +85,7 @@ private:
 		int ticket;
 		ptime expires;
 		time_duration timeout;
+		int priority;
 	};
 
 	std::list<entry> m_queue;
@@ -88,6 +94,7 @@ private:
 	int m_next_ticket;
 	int m_num_connecting;
 	int m_half_open_limit;
+	bool m_abort;
 
 	deadline_timer m_timer;
 
@@ -97,6 +104,9 @@ private:
 #ifndef NDEBUG
 	bool m_in_timeout_function;
 #endif
+#ifdef TORRENT_CONNECTION_LOGGING
+	std::ofstream m_log;
+#endif
 };
 
 }
