@@ -75,6 +75,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/intrusive_ptr_base.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/chained_buffer.hpp"
+#include "libtorrent/disk_buffer_holder.hpp"
+#include "libtorrent/bitfield.hpp"
 
 namespace libtorrent
 {
@@ -86,9 +88,21 @@ namespace libtorrent
 		struct session_impl;
 	}
 
-	struct TORRENT_EXPORT protocol_error: std::runtime_error
+	struct pending_block
 	{
-		protocol_error(const std::string& msg): std::runtime_error(msg) {};
+		pending_block(piece_block const& b): skipped(0), block(b) {}
+		int skipped;
+		// the number of times the request
+		// has been skipped by out of order blocks
+		piece_block block;
+	};
+
+	struct has_block
+	{
+		has_block(piece_block const& b): block(b) {}
+		piece_block const& block;
+		bool operator()(pending_block const& pb) const
+		{ return pb.block == block; }
 	};
 
 	class TORRENT_EXPORT peer_connection
@@ -123,6 +137,12 @@ namespace libtorrent
 			, tcp::endpoint const& remote
 			, policy::peer* peerinfo);
 
+		// this function is called after it has been constructed and properly
+		// reference counted. It is safe to call self() in this function
+		// and schedule events with references to itself (that is not safe to
+		// do in the constructor).
+		virtual void start();
+
 		virtual ~peer_connection();
 
 		void set_peer_info(policy::peer* pi)
@@ -131,7 +151,7 @@ namespace libtorrent
 		policy::peer* peer_info_struct() const
 		{ return m_peer_info; }
 
-		enum peer_speed_t { slow, medium, fast };
+		enum peer_speed_t { slow = 1, medium, fast };
 		peer_speed_t peer_speed();
 
 		void send_allowed_set();
@@ -148,7 +168,9 @@ namespace libtorrent
 
 		// this is called when the metadata is retrieved
 		// and the files has been checked
-		virtual void on_metadata() {}
+		virtual void on_metadata() {};
+
+		void on_metadata_impl();
 
 		void set_upload_limit(int limit);
 		void set_download_limit(int limit);
@@ -165,6 +187,8 @@ namespace libtorrent
 		bool on_parole() const
 		{ return peer_info_struct() && peer_info_struct()->on_parole; }
 
+		int picker_options() const;
+
 		void prefer_whole_pieces(int num)
 		{ m_prefer_whole_pieces = num; }
 
@@ -191,7 +215,13 @@ namespace libtorrent
 
 		bool is_seed() const;
 
-		bool has_timed_out() const;
+		void set_upload_only(bool u)
+		{
+			m_upload_only = u;
+			disconnect_if_redundant();
+		}
+
+		bool upload_only() const { return m_upload_only; }
 
 		// will send a keep-alive message to the peer
 		void keep_alive();
@@ -200,7 +230,7 @@ namespace libtorrent
 		void set_pid(const peer_id& pid) { m_peer_id = pid; }
 		bool has_piece(int i) const;
 
-		std::deque<piece_block> const& download_queue() const;
+		std::deque<pending_block> const& download_queue() const;
 		std::deque<piece_block> const& request_queue() const;
 		std::deque<peer_request> const& upload_queue() const;
 
@@ -224,25 +254,32 @@ namespace libtorrent
 		const stat& statistics() const { return m_statistics; }
 		void add_stat(size_type downloaded, size_type uploaded);
 
+		void calc_ip_overhead();
+
 		// is called once every second by the main loop
 		void second_tick(float tick_interval);
 
+		void timeout_requests();
+
 		boost::shared_ptr<socket_type> get_socket() const { return m_socket; }
 		tcp::endpoint const& remote() const { return m_remote; }
 
-		std::vector<bool> const& get_bitfield() const;
+		bitfield const& get_bitfield() const;
 		std::vector<int> const& allowed_fast();
 		std::vector<int> const& suggested_pieces() const { return m_suggested_pieces; }
 
+		ptime connected_time() const { return m_connect; }
+		ptime last_received() const { return m_last_receive; }
+
 		void timed_out();
 		// this will cause this peer_connection to be disconnected.
-		void disconnect();
+		void disconnect(char const* message, int error = 0);
 		bool is_disconnecting() const { return m_disconnecting; }
 
 		// this is called when the connection attempt has succeeded
 		// and the peer_connection is supposed to set m_connecting
 		// to false, and stop monitor writability
-		void on_connection_complete(asio::error_code const& e);
+		void on_connection_complete(error_code const& e);
 
 		// returns true if this connection is still waiting to
 		// finish the connection attempt
@@ -282,12 +319,24 @@ namespace libtorrent
 		bool ignore_bandwidth_limits() const
 		{ return m_ignore_bandwidth_limits; }
 
-		void set_failed() { m_failed = true; }
 		bool failed() const { return m_failed; }
 
 		int desired_queue_size() const { return m_desired_queue_size; }
 
-#ifdef TORRENT_VERBOSE_LOGGING
+		// compares this connection against the given connection
+		// for which one is more eligible for an unchoke.
+		// returns true if this is more eligible
+		bool unchoke_compare(boost::intrusive_ptr<peer_connection const> const& p) const;
+
+		// resets the byte counters that are used to measure
+		// the number of bytes transferred within unchoke cycles
+		void reset_choke_counters();
+
+		// if this peer connection is useless (neither party is
+		// interested in the other), disconnect it
+		void disconnect_if_redundant();
+
+#if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_ERROR_LOGGING
 		boost::shared_ptr<logger> m_logger;
 #endif
 
@@ -305,8 +354,9 @@ namespace libtorrent
 		void incoming_interested();
 		void incoming_not_interested();
 		void incoming_have(int piece_index);
-		void incoming_bitfield(std::vector<bool> const& bitfield);
+		void incoming_bitfield(bitfield const& bits);
 		void incoming_request(peer_request const& r);
+		void incoming_piece(peer_request const& p, disk_buffer_holder& data);
 		void incoming_piece(peer_request const& p, char const* data);
 		void incoming_piece_fragment();
 		void incoming_cancel(peer_request const& r);
@@ -322,10 +372,12 @@ namespace libtorrent
 		// the following functions appends messages
 		// to the send buffer
 		void send_choke();
-		void send_unchoke();
+		bool send_unchoke();
 		void send_interested();
 		void send_not_interested();
 
+		void snub_peer();
+
 		// adds a block to the request queue
 		void add_request(piece_block const& b);
 		// removes a block from the request queue or download queue
@@ -370,7 +422,8 @@ namespace libtorrent
 
 		// these functions are virtual to let bt_peer_connection hook into them
 		// and encrypt the content
-		virtual void send_buffer(char const* begin, int size);
+		enum message_type_flags { message_type_request = 1 };
+		virtual void send_buffer(char const* begin, int size, int flags = 0);
 		virtual buffer::interval allocate_send_buffer(int size);
 		virtual void setup_send();
 
@@ -400,6 +453,19 @@ namespace libtorrent
 		int send_buffer_capacity() const
 		{ return m_send_buffer.capacity(); }
 
+		int packet_size() const { return m_packet_size; }
+
+		bool packet_finished() const
+		{ return m_packet_size <= m_recv_pos; }
+
+#ifndef NDEBUG
+		bool piece_failed;
+#endif
+
+		// upload and download channel state
+		// enum from peer_info::bw_state
+		char m_channel_state[2];
+
 	protected:
 
 		virtual void get_specific_peer_info(peer_info& p) const = 0;
@@ -412,7 +478,7 @@ namespace libtorrent
 		virtual void write_cancel(peer_request const& r) = 0;
 		virtual void write_have(int index) = 0;
 		virtual void write_keepalive() = 0;
-		virtual void write_piece(peer_request const& r, char* buffer) = 0;
+		virtual void write_piece(peer_request const& r, disk_buffer_holder& buffer) = 0;
 		
 		virtual void write_reject_request(peer_request const& r) = 0;
 		virtual void write_allow_fast(int piece) = 0;
@@ -420,18 +486,22 @@ namespace libtorrent
 		virtual void on_connected() = 0;
 		virtual void on_tick() {}
 	
-		virtual void on_receive(asio::error_code const& error
+		virtual void on_receive(error_code const& error
 			, std::size_t bytes_transferred) = 0;
-		virtual void on_sent(asio::error_code const& error
+		virtual void on_sent(error_code const& error
 			, std::size_t bytes_transferred) = 0;
 
 #ifndef TORRENT_DISABLE_ENCRYPTION
 		buffer::interval wr_recv_buffer()
 		{
+			TORRENT_ASSERT(!m_disk_recv_buffer);
+			TORRENT_ASSERT(m_disk_recv_buffer_size == 0);
 			if (m_recv_buffer.empty()) return buffer::interval(0,0);
 			return buffer::interval(&m_recv_buffer[0]
 				, &m_recv_buffer[0] + m_recv_pos);
 		}
+
+		std::pair<buffer::interval, buffer::interval> wr_recv_buffers(int bytes);
 #endif
 		
 		buffer::const_interval receive_buffer() const
@@ -441,15 +511,11 @@ namespace libtorrent
 				, &m_recv_buffer[0] + m_recv_pos);
 		}
 
+		bool allocate_disk_receive_buffer(int disk_buffer_size);
+		char* release_disk_receive_buffer();
+		bool has_disk_receive_buffer() const { return m_disk_recv_buffer; }
 		void cut_receive_buffer(int size, int packet_size);
-
 		void reset_recv_buffer(int packet_size);
-		int packet_size() const { return m_packet_size; }
-
-		bool packet_finished() const
-		{
-			return m_packet_size <= m_recv_pos;
-		}
 
 		void setup_receive();
 
@@ -472,9 +538,9 @@ namespace libtorrent
 
 		// called from the main loop when this connection has any
 		// work to do.
-		void on_send_data(asio::error_code const& error
+		void on_send_data(error_code const& error
 			, std::size_t bytes_transferred);
-		void on_receive_data(asio::error_code const& error
+		void on_receive_data(error_code const& error
 			, std::size_t bytes_transferred);
 
 		// this is the limit on the number of outstanding requests
@@ -500,6 +566,14 @@ namespace libtorrent
 		char m_country[2];
 #endif
 
+#ifndef NDEBUG
+		boost::intrusive_ptr<peer_connection> self()
+		{
+			TORRENT_ASSERT(!m_in_constructor);
+			return intrusive_ptr_base<peer_connection>::self();
+		}
+#endif
+
 	private:
 
 		void fill_send_buffer();
@@ -507,9 +581,6 @@ namespace libtorrent
 		void on_disk_write_complete(int ret, disk_io_job const& j
 			, peer_request r, boost::shared_ptr<torrent> t);
 
-		// the timeout in seconds
-		int m_timeout;
-
 		// the time when we last got a part of a
 		// piece packet from this peer
 		ptime m_last_piece;
@@ -522,21 +593,66 @@ namespace libtorrent
 		// the time when we unchoked this peer
 		ptime m_last_unchoke;
 
-		int m_packet_size;
-		int m_recv_pos;
-		buffer m_recv_buffer;
-
-		chained_buffer m_send_buffer;
-
-		// the number of bytes we are currently reading
-		// from disk, that will be added to the send
-		// buffer as soon as they complete
-		int m_reading_bytes;
-		
 		// timeouts
 		ptime m_last_receive;
 		ptime m_last_sent;
 
+		// the time when the first entry in the
+		// request queue was requested, increased
+		// for each entry that is popped from the
+		// download queue. Used for request timeout
+		ptime m_requested;
+
+		// if the timeout is extended for the outstanding
+		// requests, this is the number of seconds it was
+		// extended.
+		int m_timeout_extend;
+
+		// a timestamp when the remote download rate
+		// was last updated
+		ptime m_remote_dl_update;
+
+		// the time when async_connect was called
+		// or when the incoming connection was established
+		ptime m_connect;
+
+		// the time when this peer sent us a not_interested message
+		// the last time.
+		ptime m_became_uninterested;
+
+		// the time when we sent a not_interested message to
+		// this peer the last time.
+		ptime m_became_uninteresting;
+
+		// the amount of data this peer has been given
+		// as free upload. This is distributed from
+		// peers from which we get free download
+		// this will be negative on a peer from which
+		// we get free download, and positive on peers
+		// that we give the free upload, to keep the balance.
+		size_type m_free_upload;
+
+		// the total payload download bytes
+		// at the last unchoke cycle. This is used to
+		// measure the number of bytes transferred during
+		// an unchoke cycle, to unchoke peers the more bytes
+		// they sent us
+		size_type m_downloaded_at_last_unchoke;
+
+#ifndef TORRENT_DISABLE_GEO_IP
+		std::string m_inet_as_name;
+#endif
+
+		buffer m_recv_buffer;
+
+		// if this peer is receiving a piece, this
+		// points to a disk buffer that the data is
+		// read into. This eliminates a memcopy from
+		// the receive buffer into the disk buffer
+		disk_buffer_holder m_disk_recv_buffer;
+
+		chained_buffer m_send_buffer;
+
 		boost::shared_ptr<socket_type> m_socket;
 		// this is the peer we're actually talking to
 		// it may not necessarily be the peer we're
@@ -549,51 +665,12 @@ namespace libtorrent
 		// until the info_hash is received. Then it's
 		// set to the torrent it belongs to.
 		boost::weak_ptr<torrent> m_torrent;
-		// is true if it was we that connected to the peer
-		// and false if we got an incoming connection
-		// could be considered: true = local, false = remote
-		bool m_active;
 
 		// remote peer's id
 		peer_id m_peer_id;
 
-		// other side says that it's interested in downloading
-		// from us.
-		bool m_peer_interested;
-
-		// the other side has told us that it won't send anymore
-		// data to us for a while
-		bool m_peer_choked;
-
-		// the peer has pieces we are interested in
-		bool m_interesting;
-
-		// we have choked the upload to the peer
-		bool m_choked;
-
-		// this is set to true if the connection timed
-		// out or closed the connection. In that
-		// case we will not try to reconnect to
-		// this peer
-		bool m_failed;
-
-		// if this is set to true, the peer will not
-		// request bandwidth from the limiter, but instead
-		// just send and receive as much as possible.
-		bool m_ignore_bandwidth_limits;
-
 		// the pieces the other end have
-		std::vector<bool> m_have_piece;
-		// this is set to true when a have_all
-		// message is received. This information
-		// is used to fill the bitmask in init()
-		bool m_have_all;
-
-		// the number of pieces this peer
-		// has. Must be the same as
-		// std::count(m_have_piece.begin(),
-		// m_have_piece.end(), true)
-		int m_num_pieces;
+		bitfield m_have_piece;
 
 		// the queue of requests we have got
 		// from this peer
@@ -605,27 +682,48 @@ namespace libtorrent
 		
 		// the queue of blocks we have requested
 		// from this peer
-		std::deque<piece_block> m_download_queue;
+		std::deque<pending_block> m_download_queue;
 		
-		// the number of request we should queue up
-		// at the remote end.
-		int m_desired_queue_size;
+		// the pieces we will send to the peer
+		// if requested (regardless of choke state)
+		std::set<int> m_accept_fast;
 
-		// the amount of data this peer has been given
-		// as free upload. This is distributed from
-		// peers from which we get free download
-		// this will be negative on a peer from which
-		// we get free download, and positive on peers
-		// that we give the free upload, to keep the balance.
-		size_type m_free_upload;
+		// the pieces the peer will send us if
+		// requested (regardless of choke state)
+		std::vector<int> m_allowed_fast;
 
-		// if this is true, this peer is assumed to handle all piece
-		// requests in fifo order. All skipped blocks are re-requested
-		// immediately instead of having a looser requirement
-		// where blocks can be sent out of order. The default is to
-		// allow non-fifo order.
-		bool m_assume_fifo;
+		// pieces that has been suggested to be
+		// downloaded from this peer
+		std::vector<int> m_suggested_pieces;
 
+		// a list of byte offsets inside the send buffer
+		// the piece requests
+		std::vector<int> m_requests_in_buffer;
+
+		// the number of pieces this peer
+		// has. Must be the same as
+		// std::count(m_have_piece.begin(),
+		// m_have_piece.end(), true)
+		int m_num_pieces;
+
+		// the timeout in seconds
+		int m_timeout;
+
+		// the size (in bytes) of the bittorrent message
+		// we're currently receiving
+		int m_packet_size;
+
+		// the number of bytes of the bittorrent payload
+		// we've received so far
+		int m_recv_pos;
+
+		int m_disk_recv_buffer_size;
+
+		// the number of bytes we are currently reading
+		// from disk, that will be added to the send
+		// buffer as soon as they complete
+		int m_reading_bytes;
+		
 		// the number of invalid piece-requests
 		// we have got from this peer. If the request
 		// queue gets empty, and there have been
@@ -635,55 +733,6 @@ namespace libtorrent
 		// by sending choke, unchoke.
 		int m_num_invalid_requests;
 
-		// this is true if this connection has been added
-		// to the list of connections that will be closed.
-		bool m_disconnecting;
-
-		// the time when this peer sent us a not_interested message
-		// the last time.
-		ptime m_became_uninterested;
-
-		// the time when we sent a not_interested message to
-		// this peer the last time.
-		ptime m_became_uninteresting;
-
-		// this is true until this socket has become
-		// writable for the first time (i.e. the
-		// connection completed). While connecting
-		// the timeout will not be triggered. This is
-		// because windows XP SP2 may delay connection
-		// attempts, which means that the connection
-		// may not even have been attempted when the
-		// time out is reached.
-		bool m_connecting;
-
-		// This is true until connect is called on the
-		// peer_connection's socket. It is false on incoming
-		// connections.
-		bool m_queued;
-
-		// these are true when there's a asynchronous write
-		// or read operation in progress. Or an asyncronous bandwidth
-		// request is in progress.
-		bool m_writing;
-		bool m_reading;
-
-		// if set to non-zero, this peer will always prefer
-		// to request entire n pieces, rather than blocks.
-		// where n is the value of this variable.
-		// if it is 0, the download rate limit setting
-		// will be used to determine if whole pieces
-		// are preferred.
-		int m_prefer_whole_pieces;
-		
-		// if this is true, the blocks picked by the piece
-		// picker will be merged before passed to the
-		// request function. i.e. subsequent blocks are
-		// merged into larger blocks. This is used by
-		// the http-downloader, to request whole pieces
-		// at a time.
-		bool m_request_large_blocks;
-		
 		// this is the priority with which this peer gets
 		// download bandwidth quota assigned to it.
 		int m_priority;
@@ -715,36 +764,117 @@ namespace libtorrent
 		// approximate peer download rate
 		int m_remote_dl_rate;
 
-		// a timestamp when the remote download rate
-		// was last updated
-		ptime m_remote_dl_update;
-
-		// the pieces we will send to the peer
-		// if requested (regardless of choke state)
-		std::set<int> m_accept_fast;
-
-		// the pieces the peer will send us if
-		// requested (regardless of choke state)
-		std::vector<int> m_allowed_fast;
-
-		// pieces that has been suggested to be
-		// downloaded from this peer
-		std::vector<int> m_suggested_pieces;
-
 		// the number of bytes send to the disk-io
 		// thread that hasn't yet been completely written.
 		int m_outstanding_writing_bytes;
 
+		// max transfer rates seen on this peer
+		int m_download_rate_peak;
+		int m_upload_rate_peak;
+
+		// estimated round trip time to this peer
+		// based on the time from when async_connect
+		// was called to when on_connection_complete
+		// was called. The rtt is specified in milliseconds
+		boost::uint16_t m_rtt;
+
+		// if set to non-zero, this peer will always prefer
+		// to request entire n pieces, rather than blocks.
+		// where n is the value of this variable.
+		// if it is 0, the download rate limit setting
+		// will be used to determine if whole pieces
+		// are preferred.
+		boost::uint8_t m_prefer_whole_pieces;
+		
+		// the number of request we should queue up
+		// at the remote end.
+		boost::uint8_t m_desired_queue_size;
+
 		// if this is true, the disconnection
 		// timestamp is not updated when the connection
 		// is closed. This means the time until we can
 		// reconnect to this peer is shorter, and likely
 		// immediate.
-		bool m_fast_reconnect;
+		bool m_fast_reconnect:1;
 		
+		// is true if it was we that connected to the peer
+		// and false if we got an incoming connection
+		// could be considered: true = local, false = remote
+		bool m_active:1;
+
+		// other side says that it's interested in downloading
+		// from us.
+		bool m_peer_interested:1;
+
+		// the other side has told us that it won't send anymore
+		// data to us for a while
+		bool m_peer_choked:1;
+
+		// the peer has pieces we are interested in
+		bool m_interesting:1;
+
+		// we have choked the upload to the peer
+		bool m_choked:1;
+
+		// this is set to true if the connection timed
+		// out or closed the connection. In that
+		// case we will not try to reconnect to
+		// this peer
+		bool m_failed:1;
+
+		// if this is set to true, the peer will not
+		// request bandwidth from the limiter, but instead
+		// just send and receive as much as possible.
+		bool m_ignore_bandwidth_limits:1;
+
+		// this is set to true when a have_all
+		// message is received. This information
+		// is used to fill the bitmask in init()
+		bool m_have_all:1;
+
+		// this is true if this connection has been added
+		// to the list of connections that will be closed.
+		bool m_disconnecting:1;
+
+		// this is true until this socket has become
+		// writable for the first time (i.e. the
+		// connection completed). While connecting
+		// the timeout will not be triggered. This is
+		// because windows XP SP2 may delay connection
+		// attempts, which means that the connection
+		// may not even have been attempted when the
+		// time out is reached.
+		bool m_connecting:1;
+
+		// This is true until connect is called on the
+		// peer_connection's socket. It is false on incoming
+		// connections.
+		bool m_queued:1;
+
+		// if this is true, the blocks picked by the piece
+		// picker will be merged before passed to the
+		// request function. i.e. subsequent blocks are
+		// merged into larger blocks. This is used by
+		// the http-downloader, to request whole pieces
+		// at a time.
+		bool m_request_large_blocks:1;
+		
+		// set to true when this peer is only uploading
+		bool m_upload_only:1;
+
+		// set to true when a piece request times out. The
+		// result is that the desired pending queue size
+		// is set to 1
+		bool m_snubbed:1;
+
+		// this is set to true once the bitfield is received
+		bool m_bitfield_received:1;
+
 #ifndef NDEBUG
 	public:
-		bool m_in_constructor;
+		bool m_in_constructor:1;
+		bool m_disconnect_started:1;
+		bool m_initialized:1;
 #endif
 	};
 }
