@@ -85,24 +85,20 @@ namespace libtorrent
 		// the tracker, pex, lsd or dht.
 		policy::peer* peer_from_tracker(const tcp::endpoint& remote, const peer_id& pid
 			, int source, char flags);
-		void update_peer_port(int port, policy::peer* p, int src);
+
+		// false means duplicate connection
+		bool update_peer_port(int port, policy::peer* p, int src);
 
 		// called when an incoming connection is accepted
-		void new_connection(peer_connection& c);
+		// false means the connection was refused or failed
+		bool new_connection(peer_connection& c);
 
 		// the given connection was just closed
-		void connection_closed(const peer_connection& c) throw();
+		void connection_closed(const peer_connection& c);
 
 		// the peer has got at least one interesting piece
 		void peer_is_interesting(peer_connection& c);
 
-		void piece_finished(int index, bool successfully_verified);
-
-		// the peer choked us
-		void choked(peer_connection& c);
-
-		int count_choked() const;
-
 		// the peer unchoked us
 		void unchoked(peer_connection& c);
 
@@ -123,72 +119,14 @@ namespace libtorrent
 		struct peer
 		{
 			enum connection_type { not_connectable, connectable };
-
 			peer(tcp::endpoint const& ip, connection_type t, int src);
 
 			size_type total_download() const;
 			size_type total_upload() const;
 
-			// the ip/port pair this peer is or was connected on
-			// if it was a remote (incoming) connection, type is
-			// set thereafter. If it was a peer we got from the
-			// tracker, type is set to local_connection.
-			tcp::endpoint ip;
-			connection_type type;
-
-#ifndef TORRENT_DISABLE_ENCRYPTION
-			// Hints encryption support of peer. Only effective for
-			// and when the outgoing encryption policy allows both
-			// encrypted and non encrypted connections
-			// (pe_settings::out_enc_policy == enabled). The initial
-			// state of this flag determines the initial connection
-			// attempt type (true = encrypted, false = standard).
-			// This will be toggled everytime either an encrypted or
-			// non-encrypted handshake fails.
-			bool pe_support;
-#endif
-			// the number of failed connection attempts this peer has
-			int failcount;
-
-			// the number of times this peer has been
-			// part of a piece that failed the hash check
-			int hashfails;
-
-			// this is true if the peer is a seed
-			bool seed;
-
-			int fast_reconnects;
-
-			// true if this peer currently is unchoked
-			// because of an optimistic unchoke.
-			// when the optimistic unchoke is moved to
-			// another peer, this peer will be choked
-			// if this is true
-			bool optimistically_unchoked;
-
-			// the time when this peer was optimistically unchoked
-			// the last time.
-			libtorrent::ptime last_optimistically_unchoked;
-
-			// the time when the peer connected to us
-			// or disconnected if it isn't connected right now
-			libtorrent::ptime connected;
-
-			// for every valid piece we receive where this
-			// peer was one of the participants, we increase
-			// this value. For every invalid piece we receive
-			// where this peer was a participant, we decrease
-			// this value. If it sinks below a threshold, its
-			// considered a bad peer and will be banned.
-			int trust_points;
-
-			// if this is true, the peer has previously participated
-			// in a piece that failed the piece hash check. This will
-			// put the peer on parole and only request entire pieces.
-			// if a piece pass that was partially requested from this
-			// peer it will leave parole mode and continue download
-			// pieces as normal peers.
-			bool on_parole;
+			tcp::endpoint ip() const { return tcp::endpoint(addr, port); }
+			void set_ip(tcp::endpoint const& endp)
+			{ addr = endp.address(); port = endp.port(); }
 
 			// this is the accumulated amount of
 			// uploaded and downloaded data to this
@@ -202,16 +140,102 @@ namespace libtorrent
 			size_type prev_amount_upload;
 			size_type prev_amount_download;
 
-			// is set to true if this peer has been banned
-			bool banned;
+			// the ip address this peer is or was connected on
+			address addr;
 
-			// a bitmap combining the peer_source flags
-			// from peer_info.
-			int source;
+			// the time when this peer was optimistically unchoked
+			// the last time.
+			libtorrent::ptime last_optimistically_unchoked;
+
+			// the time when the peer connected to us
+			// or disconnected if it isn't connected right now
+			libtorrent::ptime connected;
 
 			// if the peer is connected now, this
 			// will refer to a valid peer_connection
 			peer_connection* connection;
+
+#ifndef TORRENT_DISABLE_GEO_IP
+#ifndef NDEBUG
+			// only used in debug mode to assert that
+			// the first entry in the AS pair keeps the same
+			boost::uint16_t inet_as_num;
+#endif
+			// The AS this peer belongs to
+			std::pair<const int, int>* inet_as;
+#endif
+
+			// the port this peer is or was connected on
+			boost::uint16_t port;
+
+			// the number of failed connection attempts
+			// this peer has
+			boost::uint8_t failcount;
+
+			// for every valid piece we receive where this
+			// peer was one of the participants, we increase
+			// this value. For every invalid piece we receive
+			// where this peer was a participant, we decrease
+			// this value. If it sinks below a threshold, its
+			// considered a bad peer and will be banned.
+			boost::int8_t trust_points;
+
+			// a bitmap combining the peer_source flags
+			// from peer_info.
+			boost::uint8_t source;
+
+			// the number of times this peer has been
+			// part of a piece that failed the hash check
+			boost::uint8_t hashfails;
+
+			// type specifies if the connection was incoming
+			// or outgoing. If we ever saw this peer as connectable
+			// it will remain as connectable
+			unsigned type:4;
+
+			// the number of times we have allowed a fast
+			// reconnect for this peer.
+			unsigned fast_reconnects:4;
+
+#ifndef TORRENT_DISABLE_ENCRYPTION
+			// Hints encryption support of peer. Only effective
+			// for and when the outgoing encryption policy
+			// allows both encrypted and non encrypted
+			// connections (pe_settings::out_enc_policy
+			// == enabled). The initial state of this flag
+			// determines the initial connection attempt
+			// type (true = encrypted, false = standard).
+			// This will be toggled everytime either an
+			// encrypted or non-encrypted handshake fails.
+			bool pe_support:1;
+#endif
+			// true if this peer currently is unchoked
+			// because of an optimistic unchoke.
+			// when the optimistic unchoke is moved to
+			// another peer, this peer will be choked
+			// if this is true
+			bool optimistically_unchoked:1;
+
+			// this is true if the peer is a seed
+			bool seed:1;
+
+			// if this is true, the peer has previously
+			// participated in a piece that failed the piece
+			// hash check. This will put the peer on parole
+			// and only request entire pieces. If a piece pass
+			// that was partially requested from this peer it
+			// will leave parole mode and continue download
+			// pieces as normal peers.
+			bool on_parole:1;
+
+			// is set to true if this peer has been banned
+			bool banned:1;
+
+#ifndef TORRENT_DISABLE_DHT
+			// this is set to true when this peer as been
+			// pinged by the DHT
+			bool added_to_dht:1;
+#endif
 		};
 
 		int num_peers() const { return m_peers.size(); }
@@ -224,37 +248,50 @@ namespace libtorrent
 		const_iterator end_peer() const { return m_peers.end(); }
 
 		bool connect_one_peer();
-		bool disconnect_one_peer();
+
+		bool has_peer(policy::peer const* p) const;
+
+		int num_seeds() const { return m_num_seeds; }
+		int num_connect_candidates() const { return m_num_connect_candidates; }
+		void recalculate_connect_candidates()
+		{
+			if (m_num_connect_candidates == 0)
+				m_num_connect_candidates = 1;
+		}
+
+		void erase_peer(iterator i);
 
 	private:
-/*
-		bool unchoke_one_peer();
-		void choke_one_peer();
-		iterator find_choke_candidate();
-		iterator find_unchoke_candidate();
 
-		// the seed prefix means that the
-		// function is used while seeding.
-		bool seed_unchoke_one_peer();
-		void seed_choke_one_peer();
-		iterator find_seed_choke_candidate();
-		iterator find_seed_unchoke_candidate();
-*/
-		iterator find_disconnect_candidate();
+		bool compare_peer(policy::peer const& lhs, policy::peer const& rhs
+			, address const& external_ip) const;
+
 		iterator find_connect_candidate();
 
+		bool is_connect_candidate(peer const& p, bool finished);
+
 		std::multimap<address, peer> m_peers;
 
+		// since the peer list can grow too large
+		// to scan all of it, start at this iterator
+		iterator m_round_robin;
+
 		torrent* m_torrent;
 
 		// free download we have got that hasn't
 		// been distributed yet.
 		size_type m_available_free_upload;
 
-		// if there is a connection limit,
-		// we disconnect one peer every minute in hope of
-		// establishing a connection with a better peer
-//		ptime m_last_optimistic_disconnect;
+		// The number of peers in our peer list
+		// that are connect candidates. i.e. they're
+		// not already connected and they have not
+		// yet reached their max try count and they
+		// have the connectable state (we have a listen
+		// port for them).
+		int m_num_connect_candidates;
+
+		// the number of seeds in the peer list
+		int m_num_seeds;
 	};
 
 }
