@@ -35,8 +35,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <algorithm>
 #include <iostream>
 #include <iomanip>
+#include <boost/bind.hpp>
 #include "libtorrent/entry.hpp"
 #include "libtorrent/config.hpp"
+#include "libtorrent/escape_string.hpp"
 
 #if defined(_MSC_VER)
 namespace std
@@ -143,9 +145,38 @@ namespace libtorrent
 	}
 #endif
 
+	entry::entry()
+		: m_type(undefined_t)
+	{
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
+	}
+
+	entry::entry(data_type t)
+		: m_type(undefined_t)
+	{
+		construct(t);
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
+	}
+
+	entry::entry(const entry& e)
+		: m_type(undefined_t)
+	{
+		copy(e);
+#ifndef NDEBUG
+		m_type_queried = e.m_type_queried;
+#endif
+	}
+
 	entry::entry(dictionary_type const& v)
 		: m_type(undefined_t)
 	{
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 		new(data) dictionary_type(v);
 		m_type = dictionary_t;
 	}
@@ -153,6 +184,9 @@ namespace libtorrent
 	entry::entry(string_type const& v)
 		: m_type(undefined_t)
 	{
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 		new(data) string_type(v);
 		m_type = string_t;
 	}
@@ -160,6 +194,9 @@ namespace libtorrent
 	entry::entry(list_type const& v)
 		: m_type(undefined_t)
 	{
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 		new(data) list_type(v);
 		m_type = list_t;
 	}
@@ -167,6 +204,9 @@ namespace libtorrent
 	entry::entry(integer_type const& v)
 		: m_type(undefined_t)
 	{
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 		new(data) integer_type(v);
 		m_type = int_t;
 	}
@@ -176,6 +216,9 @@ namespace libtorrent
 		destruct();
 		new(data) dictionary_type(v);
 		m_type = dictionary_t;
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 	}
 
 	void entry::operator=(string_type const& v)
@@ -183,6 +226,9 @@ namespace libtorrent
 		destruct();
 		new(data) string_type(v);
 		m_type = string_t;
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 	}
 
 	void entry::operator=(list_type const& v)
@@ -190,6 +236,9 @@ namespace libtorrent
 		destruct();
 		new(data) list_type(v);
 		m_type = list_t;
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 	}
 
 	void entry::operator=(integer_type const& v)
@@ -197,6 +246,9 @@ namespace libtorrent
 		destruct();
 		new(data) integer_type(v);
 		m_type = int_t;
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 	}
 
 	bool entry::operator==(entry const& e) const
@@ -236,16 +288,17 @@ namespace libtorrent
 			new (data) dictionary_type;
 			break;
 		default:
-			TORRENT_ASSERT(m_type == undefined_t);
-			m_type = undefined_t;
-			return;
+			TORRENT_ASSERT(t == undefined_t);
 		}
 		m_type = t;
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 	}
 
 	void entry::copy(entry const& e)
 	{
-		switch(e.m_type)
+		switch (e.type())
 		{
 		case int_t:
 			new(data) integer_type(e.integer());
@@ -260,10 +313,12 @@ namespace libtorrent
 			new (data) dictionary_type(e.dict());
 			break;
 		default:
-			m_type = undefined_t;
-			return;
+			TORRENT_ASSERT(e.type() == undefined_t);
 		}
-		m_type = e.m_type;
+		m_type = e.type();
+#ifndef NDEBUG
+		m_type_queried = true;
+#endif
 	}
 
 	void entry::destruct()
@@ -287,6 +342,9 @@ namespace libtorrent
 			break;
 		}
 		m_type = undefined_t;
+#ifndef NDEBUG
+		m_type_queried = false;
+#endif
 	}
 
 	void entry::swap(entry& e)
@@ -315,21 +373,8 @@ namespace libtorrent
 						break;
 					}
 				}
-				if (binary_string)
-				{
-					os.unsetf(std::ios_base::dec);
-					os.setf(std::ios_base::hex);
-					for (std::string::const_iterator i = string().begin(); i != string().end(); ++i)
-						os << std::setfill('0') << std::setw(2)
-							<< static_cast<unsigned int>((unsigned char)*i);
-					os.unsetf(std::ios_base::hex);
-					os.setf(std::ios_base::dec);
-					os << "\n";
-				}
-				else
-				{
-					os << string() << "\n";
-				}
+				if (binary_string) os << to_hex(string()) << "\n";
+				else os << string() << "\n";
 			} break;
 		case list_t:
 			{
@@ -344,8 +389,21 @@ namespace libtorrent
 				os << "dictionary\n";
 				for (dictionary_type::const_iterator i = dict().begin(); i != dict().end(); ++i)
 				{
+					bool binary_string = false;
+					for (std::string::const_iterator k = i->first.begin(); k != i->first.end(); ++k)
+					{
+						if (!std::isprint(static_cast<unsigned char>(*k)))
+						{
+							binary_string = true;
+							break;
+						}
+					}
 					for (int j = 0; j < indent+1; ++j) os << " ";
-					os << "[" << i->first << "]";
+					os << "[";
+					if (binary_string) os << to_hex(i->first);
+					else os << i->first;
+					os << "]";
+
 					if (i->second.type() != entry::string_t
 						&& i->second.type() != entry::int_t)
 						os << "\n";
