@@ -30,9 +30,10 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#include <boost/version.hpp>
 #include "libtorrent/pch.hpp"
-
 #include "libtorrent/file_pool.hpp"
+#include "libtorrent/error_code.hpp"
 
 #include <iostream>
 
@@ -41,7 +42,8 @@ namespace libtorrent
 	using boost::multi_index::nth_index;
 	using boost::multi_index::get;
 
-	boost::shared_ptr<file> file_pool::open_file(void* st, fs::path const& p, file::open_mode m)
+	boost::shared_ptr<file> file_pool::open_file(void* st, fs::path const& p
+		, file::open_mode m, error_code& ec)
 	{
 		TORRENT_ASSERT(st != 0);
 		TORRENT_ASSERT(p.is_complete());
@@ -59,8 +61,10 @@ namespace libtorrent
 			{
 				// this means that another instance of the storage
 				// is using the exact same file.
-				throw file_error("torrent uses the same file as another torrent "
-					"(" + p.string() + ")");
+#if BOOST_VERSION >= 103500
+				ec = error_code(errors::file_collision, libtorrent_category);
+#endif
+				return boost::shared_ptr<file>();
 			}
 
 			e.key = st;
@@ -70,8 +74,13 @@ namespace libtorrent
 				// the new read/write privilages
 				i->file_ptr.reset();
 				TORRENT_ASSERT(e.file_ptr.unique());
-				e.file_ptr.reset();
-				e.file_ptr.reset(new file(p, m));
+				e.file_ptr->close();
+				if (!e.file_ptr->open(p, m, ec))
+				{
+					m_files.erase(i);
+					return boost::shared_ptr<file>();
+				}
+				TORRENT_ASSERT(e.file_ptr->is_open());
 				e.mode = m;
 			}
 			pt.replace(i, e);
@@ -89,14 +98,33 @@ namespace libtorrent
 			TORRENT_ASSERT(lt.size() == 1 || (i->last_use <= boost::next(i)->last_use));
 			lt.erase(i);
 		}
-		lru_file_entry e(boost::shared_ptr<file>(new file(p, m)));
+		lru_file_entry e;
+		e.file_ptr.reset(new (std::nothrow)file);
+		if (!e.file_ptr)
+		{
+			ec = error_code(ENOMEM, get_posix_category());
+			return e.file_ptr;
+		}
+		if (!e.file_ptr->open(p, m, ec))
+			return boost::shared_ptr<file>();
 		e.mode = m;
 		e.key = st;
 		e.file_path = p;
 		pt.insert(e);
+		TORRENT_ASSERT(e.file_ptr->is_open());
 		return e.file_ptr;
 	}
 
+	void file_pool::release(fs::path const& p)
+	{
+		boost::mutex::scoped_lock l(m_mutex);
+
+		typedef nth_index<file_set, 0>::type path_view;
+		path_view& pt = get<0>(m_files);
+		path_view::iterator i = pt.find(p);
+		if (i != pt.end()) pt.erase(i);
+	}
+
 	void file_pool::release(void* st)
 	{
 		boost::mutex::scoped_lock l(m_mutex);
