@@ -31,60 +31,164 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/http_connection.hpp"
+#include "libtorrent/escape_string.hpp"
+#include "libtorrent/instantiate_connection.hpp"
+#include "libtorrent/gzip.hpp"
+#include "libtorrent/parse_url.hpp"
+#include "libtorrent/socket.hpp"
+#include "libtorrent/connection_queue.hpp"
 
 #include <boost/bind.hpp>
 #include <boost/lexical_cast.hpp>
-#include <asio/ip/tcp.hpp>
 #include <string>
+#include <algorithm>
 
 using boost::bind;
 
-namespace libtorrent
-{
+namespace libtorrent {
 
-	enum { max_bottled_buffer = 1024 * 1024 };
+enum { max_bottled_buffer = 1024 * 1024 };
 
-void http_connection::get(std::string const& url, time_duration timeout
-	, int handle_redirects)
+
+void http_connection::get(std::string const& url, time_duration timeout, int prio
+	, proxy_settings const* ps, int handle_redirects, std::string const& user_agent
+	, address const& bind_addr)
 {
 	std::string protocol;
 	std::string auth;
 	std::string hostname;
 	std::string path;
+	char const* error;
 	int port;
-	boost::tie(protocol, auth, hostname, port, path) = parse_url_components(url);
+
+	boost::tie(protocol, auth, hostname, port, path, error)
+		= parse_url_components(url);
+
+	if (error)
+	{
+		callback(asio::error::socket_type_not_supported);
+		return;
+	}
+
+	TORRENT_ASSERT(prio >= 0 && prio < 2);
+
+	bool ssl = false;
+	if (protocol == "https") ssl = true;
+#ifndef TORRENT_USE_OPENSSL
+	if (ssl)
+	{
+		callback(asio::error::socket_type_not_supported);
+		return;
+	}
+#endif
+	
 	std::stringstream headers;
-	headers << "GET " << path << " HTTP/1.0\r\n"
-		"Host:" << hostname <<
-		"\r\nConnection: close\r\n";
+	if (ps && (ps->type == proxy_settings::http
+		|| ps->type == proxy_settings::http_pw)
+		&& !ssl)
+	{
+		// if we're using an http proxy and not an ssl
+		// connection, just do a regular http proxy request
+		headers << "GET " << url << " HTTP/1.0\r\n";
+		if (ps->type == proxy_settings::http_pw)
+			headers << "Proxy-Authorization: Basic " << base64encode(
+				ps->username + ":" + ps->password) << "\r\n";
+		hostname = ps->hostname;
+		port = ps->port;
+		ps = 0;
+	}
+	else
+	{
+		headers << "GET " << path << " HTTP/1.0\r\n"
+			"Host:" << hostname << "\r\n";
+	}
+
 	if (!auth.empty())
 		headers << "Authorization: Basic " << base64encode(auth) << "\r\n";
-	headers << "\r\n";
+
+	if (!user_agent.empty())
+		headers << "User-Agent: " << user_agent << "\r\n";
+	
+	headers <<
+		"Connection: close\r\n"
+		"Accept-Encoding: gzip\r\n"
+		"\r\n";
+
 	sendbuffer = headers.str();
 	m_url = url;
- 	start(hostname, boost::lexical_cast<std::string>(port), timeout, handle_redirects);
+	start(hostname, boost::lexical_cast<std::string>(port), timeout, prio
+		, ps, ssl, handle_redirects, bind_addr);
 }
 
 void http_connection::start(std::string const& hostname, std::string const& port
- 	, time_duration timeout, int handle_redirects)
+	, time_duration timeout, int prio, proxy_settings const* ps, bool ssl, int handle_redirects
+	, address const& bind_addr)
 {
- 	m_redirects = handle_redirects;
+	TORRENT_ASSERT(prio >= 0 && prio < 2);
+
+	m_redirects = handle_redirects;
+	if (ps) m_proxy = *ps;
+
 	m_timeout = timeout;
-	m_timer.expires_from_now(m_timeout);
+	error_code ec;
+	m_timer.expires_from_now(m_timeout, ec);
 	m_timer.async_wait(bind(&http_connection::on_timeout
 		, boost::weak_ptr<http_connection>(shared_from_this()), _1));
 	m_called = false;
- 	m_parser.reset();
- 	m_recvbuffer.clear();
- 	m_read_pos = 0;
-	if (m_sock.is_open() && m_hostname == hostname && m_port == port)
+	m_parser.reset();
+	m_recvbuffer.clear();
+	m_read_pos = 0;
+	m_priority = prio;
+
+	if (ec)
 	{
-		asio::async_write(m_sock, asio::buffer(sendbuffer)
+		callback(ec);
+		return;
+	}
+
+	if (m_sock.is_open() && m_hostname == hostname && m_port == port
+		&& m_ssl == ssl && m_bind_addr == bind_addr)
+	{
+		async_write(m_sock, asio::buffer(sendbuffer)
 			, bind(&http_connection::on_write, shared_from_this(), _1));
 	}
 	else
 	{
-		m_sock.close();
+		m_ssl = ssl;
+		m_bind_addr = bind_addr;
+		error_code ec;
+		m_sock.close(ec);
+
+#ifdef TORRENT_USE_OPENSSL
+		if (m_ssl)
+		{
+			m_sock.instantiate<ssl_stream<socket_type> >(m_resolver.get_io_service());
+			ssl_stream<socket_type>& s = m_sock.get<ssl_stream<socket_type> >();
+			bool ret = instantiate_connection(m_resolver.get_io_service(), m_proxy, s.next_layer());
+			TORRENT_ASSERT(ret);
+		}
+		else
+		{
+			m_sock.instantiate<socket_type>(m_resolver.get_io_service());
+			bool ret = instantiate_connection(m_resolver.get_io_service()
+				, m_proxy, m_sock.get<socket_type>());
+			TORRENT_ASSERT(ret);
+		}
+#else
+		bool ret = instantiate_connection(m_resolver.get_io_service(), m_proxy, m_sock);
+		TORRENT_ASSERT(ret);
+#endif
+		if (m_bind_addr != address_v4::any())
+		{
+			error_code ec;
+			m_sock.bind(tcp::endpoint(m_bind_addr, 0), ec);
+			if (ec)
+			{
+				callback(ec);
+				return;
+			}
+		}
+
 		tcp::resolver::query query(hostname, port);
 		m_resolver.async_resolve(query, bind(&http_connection::on_resolve
 			, shared_from_this(), _1, _2));
@@ -98,49 +202,63 @@ void http_connection::on_connect_timeout()
 	if (m_connection_ticket > -1) m_cc.done(m_connection_ticket);
 	m_connection_ticket = -1;
 
-	callback(asio::error::timed_out);
-	close();
+	if (!m_endpoints.empty())
+	{
+		error_code ec;
+		m_sock.close(ec);
+	} 
+	else
+	{ 
+		callback(asio::error::timed_out);
+		close();
+	}
 }
 
 void http_connection::on_timeout(boost::weak_ptr<http_connection> p
-	, asio::error_code const& e)
+	, error_code const& e)
 {
 	boost::shared_ptr<http_connection> c = p.lock();
 	if (!c) return;
-	if (c->m_connection_ticket > -1) c->m_cc.done(c->m_connection_ticket);
-	c->m_connection_ticket = -1;
 
 	if (e == asio::error::operation_aborted) return;
 
 	if (c->m_last_receive + c->m_timeout < time_now())
 	{
-		c->callback(asio::error::timed_out);
-		c->close();
+		if (c->m_connection_ticket > -1 && !c->m_endpoints.empty())
+		{
+			error_code ec;
+			c->m_sock.close(ec);
+			c->m_timer.expires_at(c->m_last_receive + c->m_timeout, ec);
+			c->m_timer.async_wait(bind(&http_connection::on_timeout, p, _1));
+		}
+		else
+		{
+			c->callback(asio::error::timed_out);
+			c->close();
+		}
 		return;
 	}
 
 	if (!c->m_sock.is_open()) return;
-
-	c->m_timer.expires_at(c->m_last_receive + c->m_timeout);
+	error_code ec;
+	c->m_timer.expires_at(c->m_last_receive + c->m_timeout, ec);
 	c->m_timer.async_wait(bind(&http_connection::on_timeout, p, _1));
 }
 
 void http_connection::close()
 {
-	m_timer.cancel();
-	m_limiter_timer.cancel();
-	m_sock.close();
+	error_code ec;
+	m_timer.cancel(ec);
+	m_limiter_timer.cancel(ec);
+	m_sock.close(ec);
 	m_hostname.clear();
 	m_port.clear();
-
-	if (m_connection_ticket > -1) m_cc.done(m_connection_ticket);
-	m_connection_ticket = -1;
-
 	m_handler.clear();
+	m_abort = true;
 }
 
-void http_connection::on_resolve(asio::error_code const& e
-		, tcp::resolver::iterator i)
+void http_connection::on_resolve(error_code const& e
+	, tcp::resolver::iterator i)
 {
 	if (e)
 	{
@@ -149,53 +267,100 @@ void http_connection::on_resolve(asio::error_code const& e
 		return;
 	}
 	TORRENT_ASSERT(i != tcp::resolver::iterator());
-	m_cc.enqueue(bind(&http_connection::connect, shared_from_this(), _1, *i)
+
+	std::transform(i, tcp::resolver::iterator(), std::back_inserter(m_endpoints)
+		, boost::bind(&tcp::resolver::iterator::value_type::endpoint, _1));
+
+	// The following statement causes msvc to crash (ICE). Since it's not
+	// necessary in the vast majority of cases, just ignore the endpoint
+	// order for windows
+#if !defined _MSC_VER || _MSC_VER > 1310
+	// sort the endpoints so that the ones with the same IP version as our
+	// bound listen socket are first. So that when contacting a tracker,
+	// we'll talk to it from the same IP that we're listening on
+	std::partition(m_endpoints.begin(), m_endpoints.end()
+		, boost::bind(&address::is_v4, boost::bind(&tcp::endpoint::address, _1)) == m_bind_addr.is_v4());
+#endif
+
+	queue_connect();
+}
+
+void http_connection::queue_connect()
+{
+	TORRENT_ASSERT(!m_endpoints.empty());
+	tcp::endpoint target = m_endpoints.front();
+	m_endpoints.pop_front();
+
+	m_cc.enqueue(bind(&http_connection::connect, shared_from_this(), _1, target)
 		, bind(&http_connection::on_connect_timeout, shared_from_this())
-		, m_timeout);
+		, m_timeout, m_priority);
 }
 
 void http_connection::connect(int ticket, tcp::endpoint target_address)
 {
 	m_connection_ticket = ticket;
 	m_sock.async_connect(target_address, boost::bind(&http_connection::on_connect
-		, shared_from_this(), _1/*, ++i*/));
+		, shared_from_this(), _1));
 }
 
-void http_connection::on_connect(asio::error_code const& e
-	/*, tcp::resolver::iterator i*/)
+void http_connection::on_connect(error_code const& e)
 {
+	if (m_connection_ticket >= 0)
+	{
+		m_cc.done(m_connection_ticket);
+		m_connection_ticket = -1;
+	}
+
+	m_last_receive = time_now();
 	if (!e)
 	{ 
-		m_last_receive = time_now();
 		if (m_connect_handler) m_connect_handler(*this);
-		asio::async_write(m_sock, asio::buffer(sendbuffer)
+		async_write(m_sock, asio::buffer(sendbuffer)
 			, bind(&http_connection::on_write, shared_from_this(), _1));
 	}
-/*	else if (i != tcp::resolver::iterator())
+	else if (!m_endpoints.empty() && !m_abort)
 	{
 		// The connection failed. Try the next endpoint in the list.
-		m_sock.close();
-		m_cc.enqueue(bind(&http_connection::connect, shared_from_this(), _1, *i)
-			, bind(&http_connection::on_connect_timeout, shared_from_this())
-			, m_timeout);
+		error_code ec;
+		m_sock.close(ec);
+		queue_connect();
 	} 
-*/	else
+	else
 	{ 
 		callback(e);
 		close();
 	}
 }
 
-void http_connection::callback(asio::error_code const& e, char const* data, int size)
+void http_connection::callback(error_code const& e, char const* data, int size)
 {
 	if (!m_bottled || !m_called)
 	{
+		std::vector<char> buf;
+		if (m_bottled && m_parser.header_finished())
+		{
+			std::string const& encoding = m_parser.header("content-encoding");
+			if (encoding == "gzip" || encoding == "x-gzip")
+			{
+				std::string error;
+				if (inflate_gzip(data, size, buf, max_bottled_buffer, error))
+				{
+					if (m_handler) m_handler(asio::error::fault, m_parser, data, size, *this);
+					close();
+					return;
+				}
+				data = &buf[0];
+				size = int(buf.size());
+			}
+		}
 		m_called = true;
+		error_code ec;
+		m_timer.cancel(ec);
 		if (m_handler) m_handler(e, m_parser, data, size, *this);
 	}
 }
 
-void http_connection::on_write(asio::error_code const& e)
+void http_connection::on_write(error_code const& e)
 {
 	if (e)
 	{
@@ -214,7 +379,7 @@ void http_connection::on_write(asio::error_code const& e)
 		if (m_download_quota == 0)
 		{
 			if (!m_limiter_timer_active)
-				on_assign_bandwidth(asio::error_code());
+				on_assign_bandwidth(error_code());
 			return;
 		}
 	}
@@ -224,7 +389,7 @@ void http_connection::on_write(asio::error_code const& e)
 		, shared_from_this(), _1, _2));
 }
 
-void http_connection::on_read(asio::error_code const& e
+void http_connection::on_read(error_code const& e
 	, std::size_t bytes_transferred)
 {
 	if (m_rate_limit)
@@ -263,15 +428,13 @@ void http_connection::on_read(asio::error_code const& e
 	{
 		libtorrent::buffer::const_interval rcv_buf(&m_recvbuffer[0]
 			, &m_recvbuffer[0] + m_read_pos);
-		try
+		bool error = false;
+		m_parser.incoming(rcv_buf, error);
+		if (error)
 		{
-			m_parser.incoming(rcv_buf);
-		}
-		catch (std::exception&)
-		{
-			m_timer.cancel();
-			callback(asio::error::fault, 0, 0);
-			close();
+			// HTTP parse error
+			error_code ec = asio::error::fault;
+			callback(ec, 0, 0);
 			return;
 		}
 
@@ -292,14 +455,17 @@ void http_connection::on_read(asio::error_code const& e
 					return;
 				}
 
-				try
+				error_code ec;
+				m_sock.close(ec);
+				using boost::tuples::ignore;
+				char const* error;
+				boost::tie(ignore, ignore, ignore, ignore, ignore, error)
+					= parse_url_components(location);
+				if (error == 0)
 				{
-					asio::error_code ec;
-					m_sock.close(ec);
-					get(location, m_timeout, m_redirects - 1);
-					return;
+					get(location, m_timeout, m_priority, &m_proxy, m_redirects - 1);
 				}
-				catch (std::exception& e)
+				else
 				{
 					// some broken web servers send out relative paths
 					// in the location header.
@@ -316,16 +482,7 @@ void http_connection::on_read(asio::error_code const& e
 					}
 					url += location;
 
-					try
-					{
-						get(url, m_timeout, m_redirects - 1);
-					}
-					catch (std::exception& e)
-					{
-						// location header is invalid
-						callback(asio::error::fault);
-						close();
-					}
+					get(url, m_timeout, m_priority, &m_proxy, m_redirects - 1);
 				}
 				return;
 			}
@@ -343,7 +500,8 @@ void http_connection::on_read(asio::error_code const& e
 		}
 		else if (m_bottled && m_parser.finished())
 		{
-			m_timer.cancel();
+			error_code ec;
+			m_timer.cancel(ec);
 			callback(e, m_parser.get_body().begin, m_parser.get_body().left());
 		}
 	}
@@ -370,7 +528,7 @@ void http_connection::on_read(asio::error_code const& e
 		if (m_download_quota == 0)
 		{
 			if (!m_limiter_timer_active)
-				on_assign_bandwidth(asio::error_code());
+				on_assign_bandwidth(error_code());
 			return;
 		}
 	}
@@ -380,7 +538,7 @@ void http_connection::on_read(asio::error_code const& e
 		, shared_from_this(), _1, _2));
 }
 
-void http_connection::on_assign_bandwidth(asio::error_code const& e)
+void http_connection::on_assign_bandwidth(error_code const& e)
 {
 	if ((e == asio::error::operation_aborted
 		&& m_limiter_timer_active)
@@ -407,8 +565,9 @@ void http_connection::on_assign_bandwidth(asio::error_code const& e)
 		, bind(&http_connection::on_read
 		, shared_from_this(), _1, _2));
 
+	error_code ec;
 	m_limiter_timer_active = true;
-	m_limiter_timer.expires_from_now(milliseconds(250));
+	m_limiter_timer.expires_from_now(milliseconds(250), ec);
 	m_limiter_timer.async_wait(bind(&http_connection::on_assign_bandwidth
 		, shared_from_this(), _1));
 }
@@ -419,8 +578,9 @@ void http_connection::rate_limit(int limit)
 
 	if (!m_limiter_timer_active)
 	{
+		error_code ec;
 		m_limiter_timer_active = true;
-		m_limiter_timer.expires_from_now(milliseconds(250));
+		m_limiter_timer.expires_from_now(milliseconds(250), ec);
 		m_limiter_timer.async_wait(bind(&http_connection::on_assign_bandwidth
 			, shared_from_this(), _1));
 	}
