@@ -55,6 +55,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/extensions.hpp"
 #include "libtorrent/extensions/metadata_transfer.hpp"
 #include "libtorrent/alert_types.hpp"
+#include "libtorrent/buffer.hpp"
 
 namespace libtorrent { namespace
 {
@@ -107,40 +108,37 @@ namespace libtorrent { namespace
 
 		virtual void on_files_checked()
 		{
-			// if the torrent is a seed, copy the metadata from
-			// the torrent before it is deallocated
-			if (m_torrent.is_seed())
-				metadata();
+			// if the torrent is a seed, make a reference to
+			// the metadata from the torrent before it is deallocated
+			if (m_torrent.is_seed()) metadata();
 		}
 
 		virtual boost::shared_ptr<peer_plugin> new_connection(
 			peer_connection* pc);
 		
-		std::vector<char> const& metadata() const
+		buffer::const_interval metadata() const
 		{
-			if (m_metadata.empty())
+			if (!m_metadata)
 			{
-				bencode(std::back_inserter(m_metadata)
-					, m_torrent.torrent_file().create_info_metadata());
-
-				TORRENT_ASSERT(hasher(&m_metadata[0], m_metadata.size()).final()
+				m_metadata = m_torrent.torrent_file().metadata();
+				m_metadata_size = m_torrent.torrent_file().metadata_size();
+				TORRENT_ASSERT(hasher(m_metadata.get(), m_metadata_size).final()
 					== m_torrent.torrent_file().info_hash());
 			}
-			TORRENT_ASSERT(!m_metadata.empty());
-			return m_metadata;
+			return buffer::const_interval(m_metadata.get(), m_metadata.get()
+				+ m_metadata_size);
 		}
 
 		bool received_metadata(char const* buf, int size, int offset, int total_size)
 		{
 			if (m_torrent.valid_metadata()) return false;
 
-			if ((int)m_metadata.size() < total_size)
-				m_metadata.resize(total_size);
-
-			std::copy(
-				buf
-				, buf + size
-				, &m_metadata[offset]);
+			if (!m_metadata || m_metadata_size < total_size)
+			{
+				m_metadata.reset(new char[total_size]);
+				m_metadata_size = total_size;
+			}
+			std::copy(buf, buf + size, &m_metadata[offset]);
 
 			if (m_have_metadata.empty())
 				m_have_metadata.resize(256, false);
@@ -163,7 +161,7 @@ namespace libtorrent { namespace
 			if (!have_all) return false;
 
 			hasher h;
-			h.update(&m_metadata[0], (int)m_metadata.size());
+			h.update(&m_metadata[0], m_metadata_size);
 			sha1_hash info_hash = h.final();
 
 			if (info_hash != m_torrent.torrent_file().info_hash())
@@ -175,17 +173,27 @@ namespace libtorrent { namespace
 				m_metadata_progress = 0;
 				m_metadata_size = 0;
 
-				if (m_torrent.alerts().should_post(alert::info))
+				if (m_torrent.alerts().should_post<metadata_failed_alert>())
 				{
 					m_torrent.alerts().post_alert(metadata_failed_alert(
-						m_torrent.get_handle(), "invalid metadata received from swarm"));
+						m_torrent.get_handle()));
 				}
 
 				return false;
 			}
 
-			entry metadata = bdecode(m_metadata.begin(), m_metadata.end());
-			m_torrent.set_metadata(metadata);
+			lazy_entry e;
+			lazy_bdecode(m_metadata.get(), m_metadata.get() + m_metadata_size, e);
+			std::string error;
+			if (!m_torrent.set_metadata(e, error))
+			{
+				// this means the metadata is correct, since we
+				// verified it against the info-hash, but we
+				// failed to parse it. Pause the torrent
+				// TODO: Post an alert!
+				m_torrent.pause();
+				return false;
+			}
 
 			// clear the storage for the bitfield
 			std::vector<bool>().swap(m_have_metadata);
@@ -231,10 +239,10 @@ namespace libtorrent { namespace
 		// the metadata file while downloading it from
 		// peers, and while sending it.
 		// it is mutable because it's generated lazily
-		mutable std::vector<char> m_metadata;
+		mutable boost::shared_array<char> m_metadata;
 
 		int m_metadata_progress;
-		int m_metadata_size;
+		mutable int m_metadata_size;
 
 		// this is a bitfield of size 256, each bit represents
 		// a piece of the metadata. It is set to one if we
@@ -270,19 +278,17 @@ namespace libtorrent { namespace
 		}
 
 		// called when the extension handshake from the other end is received
-		virtual bool on_extension_handshake(entry const& h)
+		virtual bool on_extension_handshake(lazy_entry const& h)
 		{
-			entry const& messages = h["m"];
-			if (entry const* index = messages.find_key("LT_metadata"))
-			{
-				m_message_index = int(index->integer());
-				return true;
-			}
-			else
-			{
-				m_message_index = 0;
-				return false;
-			}
+			m_message_index = 0;
+			if (h.type() != lazy_entry::dict_t) return false;
+			lazy_entry const* messages = h.dict_find("m");
+			if (!messages || messages->type() != lazy_entry::dict_t) return false;
+
+			int index = messages->dict_find_int_value("LT_metadata", -1);
+			if (index == -1) return false;
+			m_message_index = index;
+			return true;
 		}
 
 		void write_metadata_request(std::pair<int, int> req)
@@ -299,6 +305,11 @@ namespace libtorrent { namespace
 			// abort if the peer doesn't support the metadata extension
 			if (m_message_index == 0) return;
 
+#ifdef TORRENT_VERBOSE_LOGGING
+			(*m_pc.m_logger) << time_now_string()
+				<< " ==> METADATA_REQUEST  [ start: " << start << " | size: " << size << " ]\n";
+#endif
+
 			buffer::interval i = m_pc.allocate_send_buffer(9);
 
 			detail::write_uint32(1 + 1 + 3, i.begin);
@@ -327,26 +338,40 @@ namespace libtorrent { namespace
 			if (m_torrent.valid_metadata() && !m_torrent.torrent_file().priv())
 			{
 				std::pair<int, int> offset
-					= req_to_offset(req, (int)m_tp.metadata().size());
+					= req_to_offset(req, (int)m_tp.metadata().left());
 
+				// TODO: don't allocate send buffer for the metadata part
+				// just tag it on as a separate buffer like ut_metadata
 				buffer::interval i = m_pc.allocate_send_buffer(15 + offset.second);
 
+#ifdef TORRENT_VERBOSE_LOGGING
+				(*m_pc.m_logger) << time_now_string()
+					<< " ==> METADATA [ start: " << req.first
+					<< " | size: " << req.second
+					<< " | offset: " << offset.first
+					<< " | byte_size: " << offset.second
+					<< " ]\n";
+#endif
 				// yes, we have metadata, send it
 				detail::write_uint32(11 + offset.second, i.begin);
 				detail::write_uint8(bt_peer_connection::msg_extended, i.begin);
 				detail::write_uint8(m_message_index, i.begin);
 				// means 'data packet'
 				detail::write_uint8(1, i.begin);
-				detail::write_uint32((int)m_tp.metadata().size(), i.begin);
+				detail::write_uint32((int)m_tp.metadata().left(), i.begin);
 				detail::write_uint32(offset.first, i.begin);
-				std::vector<char> const& metadata = m_tp.metadata();
-				std::copy(metadata.begin() + offset.first
-					, metadata.begin() + offset.first + offset.second, i.begin);
+				char const* metadata = m_tp.metadata().begin;
+				std::copy(metadata + offset.first
+					, metadata + offset.first + offset.second, i.begin);
 				i.begin += offset.second;
 				TORRENT_ASSERT(i.begin == i.end);
 			}
 			else
 			{
+#ifdef TORRENT_VERBOSE_LOGGING
+				(*m_pc.m_logger) << time_now_string()
+					<< " ==> DONT HAVE METADATA\n";
+#endif
 				buffer::interval i = m_pc.allocate_send_buffer(4 + 3);
 				// we don't have the metadata, reply with
 				// don't have-message
@@ -367,7 +392,10 @@ namespace libtorrent { namespace
 			if (m_message_index == 0) return false;
 
 			if (length > 500 * 1024)
-				throw protocol_error("LT_metadata message larger than 500 kB");
+			{
+				m_pc.disconnect("LT_metadata message larger than 500 kB");
+				return true;
+			}
 
 			if (body.left() < 1) return true;
 			int type = detail::read_uint8(body.begin);
@@ -380,10 +408,18 @@ namespace libtorrent { namespace
 					int start = detail::read_uint8(body.begin);
 					int size = detail::read_uint8(body.begin) + 1;
 
+#ifdef TORRENT_VERBOSE_LOGGING
+					(*m_pc.m_logger) << time_now_string()
+						<< " <== METADATA_REQUEST [ start: " << start
+						<< " | size: " << size
+						<< " ]\n";
+#endif
+
 					if (length != 3)
 					{
 						// invalid metadata request
-						throw protocol_error("invalid metadata request");
+						m_pc.disconnect("invalid metadata request");
+						return true;
 					}
 
 					write_metadata(std::make_pair(start, size));
@@ -397,14 +433,34 @@ namespace libtorrent { namespace
 					int offset = detail::read_int32(body.begin);
 					int data_size = length - 9;
 
+#ifdef TORRENT_VERBOSE_LOGGING
+					(*m_pc.m_logger) << time_now_string()
+						<< " <== METADATA [ total_size: " << total_size
+						<< " | offset: " << offset
+						<< " | data_size: " << data_size
+						<< " ]\n";
+#endif
+
 					if (total_size > 500 * 1024)
-						throw protocol_error("metadata size larger than 500 kB");
+					{
+						m_pc.disconnect("metadata size larger than 500 kB");
+						return true;
+					}
 					if (total_size <= 0)
-						throw protocol_error("invalid metadata size");
+					{
+						m_pc.disconnect("invalid metadata size");
+						return true;
+					}
 					if (offset > total_size || offset < 0)
-						throw protocol_error("invalid metadata offset");
+					{
+						m_pc.disconnect("invalid metadata offset");
+						return true;
+					}
 					if (offset + data_size > total_size)
-						throw protocol_error("invalid metadata message");
+					{
+						m_pc.disconnect("invalid metadata message");
+						return true;
+					}
 
 					m_tp.metadata_progress(total_size
 						, body.left() - m_metadata_progress);
@@ -423,10 +479,17 @@ namespace libtorrent { namespace
 				if (m_waiting_metadata_request)
 					m_tp.cancel_metadata_request(m_last_metadata_request);
 				m_waiting_metadata_request = false;
+#ifdef TORRENT_VERBOSE_LOGGING
+				(*m_pc.m_logger) << time_now_string()
+					<< " <== DONT HAVE METADATA\n";
+#endif
 				break;
 			default:
-				throw protocol_error("unknown metadata extension message: "
-					+ boost::lexical_cast<std::string>(type));
+				{
+					std::stringstream msg;
+					msg << "unknown metadata extension message: " << type;
+					m_pc.disconnect(msg.str().c_str());
+				}
 			}
 			return true;
 		}
